# from __future__ import absolute_import
from __future__ import division # confidence high

from .version import *

import stsci.tools.tester
def test(*args,**kwds):
    stsci.tools.tester.test(modname=__name__, *args, **kwds)





"""

License: http://www.stsci.edu/resources/software_hardware/pyraf/LICENSE

"""
from __future__ import division # confidence medium

from __future__ import generators
import numerixenv
numerixenv.check()

import numpy as N

BUFSIZE = 1024*1000   # 1Mb cache size

__version__ = '0.2'


def ImageIter(imgarr,bufsize=None,overlap=0,copy=0):

    imgarr = N.asarray(imgarr)
    
    if bufsize == None: bufsize = BUFSIZE

    if len(imgarr.shape) == 1:
        if copy:
            yield imgarr.copy()
        else:
            yield imgarr
    else:
        nrows = int(bufsize / (imgarr.itemsize * imgarr.shape[1]))    
        niter = int(imgarr.shape[0] / nrows) * nrows
    
        if copy:
                # Create a cache that will contain a copy of the input
                    # not just a view...
                    _cache = N.zeros((nrows,imgarr.shape[1]),dtype=imgarr.dtype.char)

        for pix in range(0,niter+1,nrows):
                    if copy:
                            _cache = imgarr[pix:pix+nrows].copy()
                            yield _cache
                    else:
                            yield imgarr[pix:pix+nrows]
                    if copy:
                            _cache *= 0

                    pix -= overlap  
             



from __future__ import division # confidence high
import sys
PY3K = sys.version_info[0] > 2

# Don't copy this as an example - use any other package is stsci_python,
# such as sample_package

pkg = [ 'stsci', 'stsci.tools', 'stsci.tools.tests' ]

setupargs = {

    'version' :         '3.1',
    'description' :     "General Use Python Tools",
    'author' :          "Warren Hack, Christopher Hanley",
    'author_email' :    "help@stsci.edu",
    'license' :         "http://www.stsci.edu/resources/software_hardware/pyraf/LICENSE",
    'platforms' :       ["Linux","Solaris","Mac OS X","Win"],
    'scripts' :         ['scripts/convertwaiveredfits','scripts/stscidocs'] ,
    'package_dir' :     { 'stsci' : 'old_stsci', 'stsci.tools' : 'lib/stsci/tools', 'stsci.tools.tests' : 'lib/stsci/tools/tests' },
    'data_files' :      [ ( 'stsci/tools/tests', [ 'lib/stsci/tools/tests/*.fits' ] ) ],
    }

if PY3K:
    # TODO: in PY3K we get: "error: can't copy 'lib/stsci/tools/tests/*.fits' "
    setupargs['data_files'] = [ ( 'stsci/tools/tests', [] ) ]



""" Read-Only Tkinter Text Widget.  This is a variation of the Tkinter Text
widget in that the text itself is not editable (it is read-only), but it allows
selection for cut/paste to other apps.  Cut-paste may currently only work
under X11.

A vastly simpler way of doing this is to use a Tkinter.Text widget and set
it to DISABLED, but then you cannot select text.
$Id$
"""
from __future__ import division # confidence high

# System level modules
import Tkinter

ALLOWED_SYMS = ('Up','Down','Left','Right','Home','End','Prior','Next', \
                'Shift_L', 'Shift_R')

class ROText(Tkinter.Text):

    def __init__(self, master, **kw):
        """  defer most of __init__ to the base class """
        self._fbto = None
        if 'focusBackTo' in kw:
            self._fbto = kw['focusBackTo']
            del kw['focusBackTo']
        Tkinter.Text.__init__(self, master, **kw)
        # override some bindings to return a "break" string
        self.bind("<Key>", self.ignoreMostKeys)
        self.bind("<Button-2>", lambda e: "break")
        self.bind("<Button-3>", lambda e: "break")
        if self._fbto:
            self.bind("<Leave>", self.mouseLeft)
        self.config(insertwidth=0)

    # disallow common insert calls, but allow a backdoor when needed
    def insert(self, index, text, *tags, **kw):
        if 'force' in kw and kw['force']:
            Tkinter.Text.insert(self, index, text, *tags)

    # disallow common delete calls, but allow a backdoor when needed
    def delete(self, start, end=None, force=False):
        if force:
            Tkinter.Text.delete(self, start, end)

    # a way to disable text manip
    def ignoreMostKeys(self, event):
        if event.keysym not in ALLOWED_SYMS:
            return "break" # have to return this string to stop the event

    def mouseLeft(self, event):
        if self._fbto:
            self._fbto.focus_set()
        return "break" # have to return this string to stop the event


# Test the above class
if __name__ == '__main__':

    import sys, time

    rot = None

    def quit():
        sys.exit()

    def clicked():
        rot.insert(Tkinter.END, "\nClicked at "+time.asctime(), force=True)
        rot.see(Tkinter.END)

    # make our test window
    top = Tkinter.Tk()
    f = Tkinter.Frame(top)

    sc = Tkinter.Scrollbar(f)
    sc.pack(side=Tkinter.RIGHT, fill=Tkinter.Y)
    rot = ROText(f, wrap=Tkinter.WORD, height=10, yscrollcommand=sc.set,
                 focusBackTo=top)
    rot.pack(side=Tkinter.TOP, fill=Tkinter.X, expand=True)
    sc.config(command=rot.yview)
    f.pack(side=Tkinter.TOP, fill=Tkinter.X)

    b = Tkinter.Button(top, text='Click Me', command=clicked)
    b.pack(side=Tkinter.TOP, fill=Tkinter.X, expand=1)

    q = Tkinter.Button(top, text='Quit', command=quit)
    q.pack(side=Tkinter.TOP)

    # start
    top.mainloop()



#!/usr/bin/env python

# $Id: fitsdiff.py 16870 2012-05-18 13:52:53Z embray $

"""fitsdiff is now a part of PyFITS--the fitsdiff in PyFITS replaces the
fitsdiff that used to be in the module.

Now this module just provides a wrapper around pyfits.diff for backwards
compatibility with the old interface in case anyone uses it.
"""

import os
import sys


from pyfits.diff import FITSDiff
from pyfits.scripts.fitsdiff import log, main


def fitsdiff(input1, input2, comment_excl_list='', value_excl_list='',
             field_excl_list='', maxdiff=10, delta=0.0, neglect_blanks=True,
             output=None):

    if isinstance(comment_excl_list, basestring):
        comment_excl_list = list_parse(comment_excl_list)

    if isinstance(value_excl_list, basestring):
        value_excl_list = list_parse(value_excl_list)

    if isinstance(field_excl_list, basestring):
        field_excl_list = list_parse(field_excl_list)

    diff = FITSDiff(input1, input2, ignore_keywords=value_excl_list,
                    ignore_comments=comment_excl_list,
                    ignore_fields=field_excl_list, numdiffs=maxdiff,
                    tolerance=delta, ignore_blanks=neglect_blanks)

    if output is None:
        output = sys.stdout

    diff.report(output)

    return diff.identical


def list_parse(name_list):
    """Parse a comma-separated list of values, or a filename (starting with @)
    containing a list value on each line.
    """

    if name_list and name_list[0] == '@':
        value = name_list[1:]
        if not os.path.exists(value):
            log.warning('The file %s does not exist' % value)
            return
        try:
            return [v.strip() for v in open(value, 'r').readlines()]
        except IOError, e:
            log.warning('reading %s failed: %s; ignoring this file' %
                        (value, e))
    else:
        return [v.strip() for v in name_list.split(',')]


if __name__ == "__main__":
    sys.exit(main())



from __future__ import division # confidence high
import os

def check_input(xxx):
    """Check if input is a Numarray Array."""
    try:
        import numarray
        return isinstance(xxx,numarray.numarraycore.NumArray)    
    except ImportError:
        pass

def check():
    """Check for running numarray version of pyfits with numpy code."""
    try:
        import pyfits
        if pyfits.__version__ < '1.1':
            raise EnvironmentError, "Pyfits 1.1 or later required, pyfits version %s detected\n" % pyfits.__version__
    except ImportError:
        pass
    try:
        if os.environ['NUMERIX'] == 'numarray':
            raise EnvironmentError, "NUMERIX/numarray environment detected; numpy environment required"
    except KeyError:
        pass

    




""" Learn basic capabilities here (e.g. can we display graphics?).
This is meant to be fast and light, having no complicated dependencies, so
that any module can fearlessly import this without adverse affects or
performance concerns.

$Id: capable.py 16505 2012-05-01 20:55:34Z sontag $
"""

from __future__ import division # confidence high

import os, sys
PY3K = sys.version_info[0] > 2


def is_darwin_and_x():
    """ Convenience function.  Returns True if is an X11-linked Python/Tkinter
    build on OSX.  This is intended to be quick and easy without further
    imports.  As a result, this relies on the assumption that on OSX, PyObjC
    is installed (only) in the Framework builds of Python. """
    if not sys.platform == 'darwin':
        return False

    return which_darwin_linkage() == "x11"


def which_darwin_linkage(force_otool_check=False):
    """ Convenience function.  Returns one of ('x11', 'aqua') in answer to the
    question of whether this is an X11-linked Python/Tkinter, or a natively
    built (framework, Aqua) one.  This is only for OSX.
    On Python 2.*, this relies on the assumption that on OSX, PyObjC
    is installed only in the Framework builds of Python.  On Python 3.*,
    this inspects the actual tkinter library binary via otool. """

    # sanity check
    assert sys.platform=='darwin', 'Incorrect usaage, not on OSX'

    # There will *usually* be PyObjC modules on sys.path on the natively-
    # linked Python. This is assumed to be always correct on Python 2.x, as
    # of 2012.  This is kludgy but quick and effective.
    if not force_otool_check:
        junk = ",".join(sys.path)
        if junk.lower().find('/pyobjc') >= 0:
            return "aqua"

    # OK, no PyObjC found.  What we do next is different per Python ver.
    if not PY3K and not force_otool_check:
        return "x11"

    # Is PY3K, use otool shell command (requires 2.7+)
    import Tkinter, subprocess
    libs = subprocess.check_output(('/usr/bin/otool', '-L', Tkinter._tkinter.__file__)).decode()
    if libs.find('/libX11.') >= 0:
        return "x11"
    else:
        return "aqua"


OF_GRAPHICS = True

if 'PYRAF_NO_DISPLAY' in os.environ or 'PYTOOLS_NO_DISPLAY' in os.environ:
    OF_GRAPHICS = False

if OF_GRAPHICS and sys.platform == 'darwin':
    #
    # On OSX, there is an AppKit error where Python itself will abort if
    # Tkinter operations (e.g. Tkinter._test() ...) are attempted when running
    # from a remote terminal.  In these situations, it is not even safe to put
    # the code in a try/except block, since the AppKit error seems to happen
    # *asynchronously* within ObjectiveC code.  See PyRAF ticket #149.
    #
    # SO, let's try a quick simple test here (only on OSX) to find out if we
    # are the "console user".  If we are not, then we don't even want to attempt
    # any windows/graphics calls.  See "console user" here:
    #     http://developer.apple.com/library/mac/#technotes/tn2083/_index.html
    # If we are the console user, we own /dev/console and can read from it.
    #
    OF_GRAPHICS = os.access("/dev/console", os.R_OK)

    # Add a double-check for remote X11 users.  We *think* this is a smaller
    # set of cases, so we do it last minute here:
    if not OF_GRAPHICS:
        # On OSX, but logged in remotely. Normally (with native build) this
        # means there are no graphics.  But, what if they're calling an
        # X11-linked Python?  Then we should allow graphics to be attempted.
        OF_GRAPHICS = is_darwin_and_x()

        # OF_GRAPHICS will be True here in only two cases (2nd should be rare):
        #    An OSX Python build linked with X11, or
        #    An OSX Python build linked natively where PyObjC was left out

# After all that, we may have decided that we want graphics.  Now
# that we know it is ok to try to import Tkinter, we can test if it
# is there.  If it is not, we are not capable of graphics.
if OF_GRAPHICS :
    try :
        import Tkinter
    except ImportError :
        TKINTER_IMPORT_FAILED = 1
        OF_GRAPHICS = False

# Using tkFileDialog from PyRAF (and maybe in straight TEAL) is crashing python
# itself on OSX only.  Allow on Linux.  Mac: use this until PyRAF #171 fixed.
OF_TKFD_IN_EPAR = True
if sys.platform == 'darwin' and OF_GRAPHICS and \
   not is_darwin_and_x(): # if framework ver
    OF_TKFD_IN_EPAR = 'TEAL_TRY_TKFD' in os.environ





#!/usr/bin/env python
from __future__ import division  # confidence high

import os
import tempfile

import numpy as np
from nose.tools import assert_true, assert_equal, assert_raises

import stsci.tools.stpyfits as stpyfits
import pyfits
from pyfits.tests import PyfitsTestCase


class TestStpyfitsFunctions(PyfitsTestCase):
    def setup(self):
        self.data_dir = os.path.dirname(__file__)
        self.temp_dir = tempfile.mkdtemp(prefix='stpyfits-test-')

    def testInfoConvienceFunction(self):
        """Test the info convience function in both the pyfits and stpyfits
           namespace."""

        assert_equal(
            stpyfits.info(self.data('o4sp040b0_raw.fits'), output=False),
            [(0, 'PRIMARY', 'PrimaryHDU', 215, (), 'int16', ''),
             (1, 'SCI', 'ImageHDU', 141, (62, 44), 'int16', ''),
             (2, 'ERR', 'ImageHDU', 71, (62, 44), 'int16', ''),
             (3, 'DQ', 'ImageHDU', 71, (62, 44), 'int16', ''),
             (4, 'SCI', 'ImageHDU', 141, (62, 44), 'int16', ''),
             (5, 'ERR', 'ImageHDU', 71, (62, 44), 'int16', ''),
             (6, 'DQ', 'ImageHDU', 71, (62, 44), 'int16', '')])


        assert_equal(
            pyfits.info(self.data('o4sp040b0_raw.fits'), output=False),
            [(0, 'PRIMARY', 'PrimaryHDU', 215, (), 'int16', ''),
             (1, 'SCI', 'ImageHDU', 141, (62, 44), 'int16', ''),
             (2, 'ERR', 'ImageHDU', 71, (), 'int16', ''),
             (3, 'DQ', 'ImageHDU', 71, (), 'int16', ''),
             (4, 'SCI', 'ImageHDU', 141, (62, 44), 'int16', ''),
             (5, 'ERR', 'ImageHDU', 71, (), 'int16', ''),
             (6, 'DQ', 'ImageHDU', 71, (), 'int16', '')])

        assert_equal(
            stpyfits.info(self.data('cdva2.fits'), output=False),
            [(0, 'PRIMARY', 'PrimaryHDU', 7, (10, 10), 'int32', '')])

        assert_equal(
            pyfits.info(self.data('cdva2.fits'), output=False),
            [(0, 'PRIMARY', 'PrimaryHDU', 7, (), 'int32', '')])


    def testOpenConvienceFunction(self):
        """Test the open convience function in both the pyfits and stpyfits
           namespace."""

        hdul = stpyfits.open(self.data('cdva2.fits'))
        hdul1 = pyfits.open(self.data('cdva2.fits'))

        assert_equal(hdul[0].header['NAXIS'], 2)
        assert_equal(hdul1[0].header['NAXIS'], 0)
        assert_equal(hdul[0].header['NAXIS1'], 10)
        assert_equal(hdul[0].header['NAXIS2'], 10)

        assert_raises(KeyError, lambda: hdul1[0].header['NAXIS1'])
        assert_raises(KeyError, lambda: hdul1[0].header['NAXIS2'])
        assert_raises(KeyError, lambda: hdul[0].header['NPIX1'])
        assert_raises(KeyError, lambda: hdul[0].header['NPIX2'])

        assert_equal(hdul1[0].header['NPIX1'], 10)
        assert_equal(hdul1[0].header['NPIX2'], 10)

        assert_true((hdul[0].data == np.ones((10, 10), dtype=np.int32)).all())

        assert_equal(hdul1[0].data, None)

        hdul.close()
        hdul1.close()

    def testGetHeaderConvienceFunction(self):
        """Test the getheader convience function in both the pyfits and
           stpyfits namespace."""

        hd = stpyfits.getheader(self.data('cdva2.fits'))
        hd1 = pyfits.getheader(self.data('cdva2.fits'))

        assert_equal(hd['NAXIS'], 2)
        assert_equal(hd1['NAXIS'], 0)
        assert_equal(hd['NAXIS1'], 10)
        assert_equal(hd['NAXIS2'], 10)

        assert_raises(KeyError, lambda: hd1['NAXIS1'])
        assert_raises(KeyError, lambda: hd1['NAXIS2'])
        assert_raises(KeyError, lambda: hd['NPIX1'])
        assert_raises(KeyError, lambda: hd['NPIX2'])

        assert_equal(hd1['NPIX1'], 10)
        assert_equal(hd1['NPIX2'], 10)

        hd = stpyfits.getheader(self.data('o4sp040b0_raw.fits'), 2)
        hd1 = pyfits.getheader(self.data('o4sp040b0_raw.fits'), 2)

        assert_equal(hd['NAXIS'], 2)
        assert_equal(hd1['NAXIS'], 0)
        assert_equal(hd['NAXIS1'], 62)
        assert_equal(hd['NAXIS2'], 44)

        assert_raises(KeyError, lambda: hd1['NAXIS1'])
        assert_raises(KeyError, lambda: hd1['NAXIS2'])
        assert_raises(KeyError, lambda: hd['NPIX1'])
        assert_raises(KeyError, lambda: hd['NPIX2'])

        assert_equal(hd1['NPIX1'], 62)
        assert_equal(hd1['NPIX2'], 44)

    def testGetDataConvienceFunction(self):
        """Test the getdata convience function in both the pyfits and
           stpyfits namespace."""

        d = stpyfits.getdata(self.data('cdva2.fits'))
        assert_true((d == np.ones((10, 10), dtype=np.int32)).all())

        assert_raises(IndexError, pyfits.getdata, self.data('cdva2.fits'))

    def testGetValConvienceFunction(self):
        """Test the getval convience function in both the pyfits and
           stpyfits namespace."""

        val = stpyfits.getval(self.data('cdva2.fits'), 'NAXIS', 0)
        val1 = pyfits.getval(self.data('cdva2.fits'), 'NAXIS', 0)
        assert_equal(val, 2)
        assert_equal(val1, 0)

    def testwritetoConvienceFunction(self):
        """Test the writeto convience function in both the pyfits and stpyfits
           namespace."""

        hdul = stpyfits.open(self.data('cdva2.fits'))
        hdul1 = pyfits.open(self.data('cdva2.fits'))

        stpyfits.writeto(self.temp('new.fits'), hdul[0].data, hdul[0].header,
                         clobber=True)
        pyfits.writeto(self.temp('new1.fits'), hdul1[0].data,hdul1[0].header,
                       clobber=True)

        hdul.close()
        hdul1.close()

        info1 = pyfits.info(self.temp('new.fits'), output=False)
        info2 = stpyfits.info(self.temp('new.fits'), output=False)
        info3 = pyfits.info(self.temp('new1.fits'), output=False)
        info4 = stpyfits.info(self.temp('new1.fits'), output=False)

        assert_equal(info1, [(0, 'PRIMARY', 'PrimaryHDU', 6, (), 'int32', '')])
        assert_equal(info2,
            [(0, 'PRIMARY', 'PrimaryHDU', 6, (10, 10), 'int32', '')])
        assert_equal(info3, [(0, 'PRIMARY', 'PrimaryHDU', 6, (), 'uint8', '')])
        assert_equal(info4,
            [(0, 'PRIMARY', 'PrimaryHDU', 6, (10, 10), 'uint8', '')])

    def testappendConvienceFunction(self):
        """Test the append convience function in both the pyfits and stpyfits
           namespace."""

        hdul = stpyfits.open(self.data('cdva2.fits'))
        hdul1 = pyfits.open(self.data('cdva2.fits'))

        stpyfits.writeto(self.temp('new.fits'), hdul[0].data, hdul[0].header,
                         clobber=True)
        pyfits.writeto(self.temp('new1.fits'), hdul1[0].data, hdul1[0].header,
                       clobber=True)

        hdu = stpyfits.ImageHDU()
        hdu1 = pyfits.ImageHDU()

        hdu.data = hdul[0].data
        hdu1.data = hdul1[0].data
        hdu.header.set('BITPIX', 32)
        hdu1.header.set('BITPIX', 32)
        hdu.header.set('NAXIS', 2)
        hdu.header.set('NAXIS1', 10, 'length of constant array axis 1',
                       after='NAXIS')
        hdu.header.set('NAXIS2', 10, 'length of constant array axis 2',
                       after='NAXIS1')
        hdu.header.set('PIXVALUE', 1, 'Constant pixel value', after='GCOUNT')
        hdu1.header.set('PIXVALUE', 1, 'Constant pixel value', after='GCOUNT')
        hdu1.header.set('NPIX1', 10, 'length of constant array axis 1',
                        after='GCOUNT')
        hdu1.header.set('NPIX2', 10, 'length of constant array axis 2',
                        after='NPIX1')
        stpyfits.append(self.temp('new.fits'), hdu.data, hdu.header)
        pyfits.append(self.temp('new1.fits'), hdu1.data, hdu1.header)

        assert_equal(stpyfits.info(self.temp('new.fits'), output=False),
            [(0, 'PRIMARY', 'PrimaryHDU', 7, (10, 10), 'int32', ''),
             (1, '', 'ImageHDU', 8, (10, 10), 'int32', '')])
        assert_equal(stpyfits.info(self.temp('new1.fits'), output=False),
            [(0, 'PRIMARY', 'PrimaryHDU', 7, (10, 10), 'uint8', ''),
             (1, '', 'ImageHDU', 8, (10, 10), 'uint8', '')])
        assert_equal(pyfits.info(self.temp('new.fits'), output=False),
            [(0, 'PRIMARY', 'PrimaryHDU', 7, (), 'int32', ''),
             (1, '', 'ImageHDU', 8, (), 'int32', '')])
        assert_equal(pyfits.info(self.temp('new1.fits'), output=False),
            [(0, 'PRIMARY', 'PrimaryHDU', 7, (), 'uint8', ''),
             (1, '', 'ImageHDU', 8, (), 'uint8', '')])

        hdul5 = stpyfits.open(self.temp('new.fits'))
        hdul6 = pyfits.open(self.temp('new1.fits'))
        assert_equal(hdul5[1].header['NAXIS'], 2)
        assert_equal(hdul6[1].header['NAXIS'], 0)
        assert_equal(hdul5[1].header['NAXIS1'], 10)
        assert_equal(hdul5[1].header['NAXIS2'], 10)

        assert_raises(KeyError, lambda: hdul6[1].header['NAXIS1'])
        assert_raises(KeyError, lambda: hdul6[1].header['NAXIS2'])
        assert_raises(KeyError, lambda: hdul5[1].header['NPIX1'])
        assert_raises(KeyError, lambda: hdul5[1].header['NPIX2'])

        assert_equal(hdul6[1].header['NPIX1'], 10)
        assert_equal(hdul6[1].header['NPIX2'], 10)

        assert_true((hdul5[1].data == np.ones((10, 10), dtype=np.int32)).all())

        assert_equal(hdul6[1].data, None)

        hdul5.close()
        hdul6.close()
        hdul.close()
        hdul1.close()

    def testupdateConvienceFunction(self):
        """Test the update convience function in both the pyfits and stpyfits
           namespace."""

        hdul = stpyfits.open(self.data('cdva2.fits'))
        hdul1 = pyfits.open(self.data('cdva2.fits'))

        stpyfits.writeto(self.temp('new.fits'), hdul[0].data, hdul[0].header,
                         clobber=True)

        hdu = stpyfits.ImageHDU()
        hdu1 = pyfits.ImageHDU()

        hdu.data = hdul[0].data
        hdu1.data = hdul1[0].data
        hdu.header.set('BITPIX', 32)
        hdu1.header.set('BITPIX', 32)
        hdu.header.set('NAXIS', 2)
        hdu.header.set('NAXIS1', 10, 'length of constant array axis 1',
                       after='NAXIS')
        hdu.header.set('NAXIS2', 10, 'length of constant array axis 2',
                       after='NAXIS1')
        hdu.header.set('PIXVALUE', 1, 'Constant pixel value', after='GCOUNT')
        hdu1.header.set('PIXVALUE', 1, 'Constant pixel value', after='GCOUNT')
        hdu1.header.set('NPIX1', 10, 'length of constant array axis 1',
                        after='GCOUNT')
        hdu1.header.set('NPIX2', 10, 'length of constant array axis 2',
                        after='NPIX1')
        stpyfits.append(self.temp('new.fits'), hdu.data, hdu.header)

        d = hdu.data * 0

        stpyfits.update(self.temp('new.fits'), d, hdu.header, 1)

        assert_equal(pyfits.info(self.temp('new.fits'), output=False),
            [(0, 'PRIMARY', 'PrimaryHDU', 7, (), 'int32', ''),
             (1, '', 'ImageHDU', 8, (), 'int32', '')])
        assert_equal(stpyfits.info(self.temp('new.fits'), output=False),
            [(0, 'PRIMARY', 'PrimaryHDU', 7, (10, 10), 'int32', ''),
             (1, '', 'ImageHDU', 8, (10, 10), 'int32', '')])

        hdul7 = stpyfits.open(self.temp('new.fits'))
        assert_equal(hdul7[1].header['NAXIS'], 2)
        assert_equal(hdul7[1].header['NAXIS1'], 10)
        assert_equal(hdul7[1].header['NAXIS2'], 10)
        assert_equal(hdul7[1].header['PIXVALUE'], 0)

        assert_raises(KeyError, lambda: hdul7[1].header['NPIX1'])
        assert_raises(KeyError, lambda: hdul7[1].header['NPIX2'])

        assert_true((hdul7[1].data ==
                     np.zeros((10, 10), dtype=np.int32)).all())

        hdul8 = pyfits.open(self.temp('new.fits'))
        assert_equal(hdul8[1].header['NAXIS'], 0)
        assert_equal(hdul8[1].header['NPIX1'], 10)
        assert_equal(hdul8[1].header['NPIX2'], 10)
        assert_equal(hdul8[1].header['PIXVALUE'], 0)

        assert_raises(KeyError, lambda: hdul8[1].header['NAXIS1'])
        assert_raises(KeyError, lambda: hdul8[1].header['NAXIS2'])

        assert_equal(hdul8[1].data, None)

        hdul7.close()
        hdul8.close()
        hdul.close()
        hdul1.close()

    def testImageHDUConstructor(self):
        """Test the ImageHDU constructor in both the pyfits and stpyfits
           namespace."""

        hdu = stpyfits.ImageHDU()
        assert_true(isinstance(hdu, stpyfits.ConstantValueImageHDU))
        hdu1 = pyfits.ImageHDU()
        assert_true(isinstance(hdu, pyfits.ImageHDU))

    def testPrimaryHDUConstructor(self):
        """Test the PrimaryHDU constructor in both the pyfits and stpyfits
           namespace.  Although stpyfits does not reimplement the
           constructor, it does add _ConstantValueImageBaseHDU to the
           inheritance hierarchy of pyfits.PrimaryHDU when accessed through the
           stpyfits namespace.  This method tests that that inheritance is
           working"""

        n = np.zeros(10)
        n = n + 1

        hdu = stpyfits.PrimaryHDU(n)
        hdu.header.set('PIXVALUE', 1.0, 'Constant pixel value', after='EXTEND')
        hdu.header.set('NAXIS', 0)
        stpyfits.writeto(self.temp('new.fits'), hdu.data, hdu.header,
                         clobber=True)
        hdul = stpyfits.open(self.temp('new.fits'))
        hdul1 = pyfits.open(self.temp('new.fits'))

        assert_equal(hdul[0].header['NAXIS'], 1)
        assert_equal(hdul[0].header['NAXIS1'], 10)
        assert_equal(hdul[0].header['PIXVALUE'], 1.0)

        assert_raises(KeyError, lambda: hdul[0].header['NPIX1'])

        assert_true((hdul[0].data == np.ones(10, dtype=np.float32)).all())

        assert_equal(hdul1[0].header['NAXIS'], 0)
        assert_equal(hdul1[0].header['NPIX1'], 10)
        assert_equal(hdul1[0].header['PIXVALUE'], 1.0)

        assert_raises(KeyError, lambda: hdul1[0].header['NAXIS1'])

        assert_equal(hdul1[0].data, None)

        hdul.close()
        hdul1.close()

    def testHDUListWritetoMethod(self):
        """Test the writeto method of HDUList in both the pyfits and stpyfits
           namespace."""

        hdu = stpyfits.PrimaryHDU()
        hdu1 = stpyfits.ImageHDU()
        hdu.data = np.zeros((10, 10), dtype=np.int32)
        hdu1.data = hdu.data + 2
        hdu.header.set('BITPIX', 32)
        hdu1.header.set('BITPIX', 32)
        hdu.header.set('NAXIS', 2)
        hdu.header.set('NAXIS1', 10, 'length of constant array axis 1',
                       after='NAXIS')
        hdu.header.set('NAXIS2', 10, 'length of constant array axis 2',
                       after='NAXIS1')
        hdu.header.set('PIXVALUE', 0, 'Constant pixel value')
        hdu1.header.set('PIXVALUE', 2, 'Constant pixel value', after='GCOUNT')
        hdu1.header.set('NAXIS', 2)
        hdu1.header.set('NAXIS1', 10, 'length of constant array axis 1',
                        after='NAXIS')
        hdu1.header.set('NAXIS2', 10, 'length of constant array axis 2',
                        after='NAXIS1')
        hdul = stpyfits.HDUList([hdu,hdu1])
        hdul.writeto(self.temp('new.fits'), clobber=True)

        assert_equal(stpyfits.info(self.temp('new.fits'), output=False),
            [(0, 'PRIMARY', 'PrimaryHDU', 7, (10, 10), 'int32', ''),
             (1, '', 'ImageHDU', 8, (10, 10), 'int32', '')])

        assert_equal(pyfits.info(self.temp('new.fits'), output=False),
            [(0, 'PRIMARY', 'PrimaryHDU', 7, (), 'int32', ''),
             (1, '', 'ImageHDU', 8, (), 'int32', '')])

        hdul1 = stpyfits.open(self.temp('new.fits'))
        hdul2 = pyfits.open(self.temp('new.fits'))

        assert_equal(hdul1[0].header['NAXIS'], 2)
        assert_equal(hdul1[0].header['NAXIS1'], 10)
        assert_equal(hdul1[0].header['NAXIS2'], 10)
        assert_equal(hdul1[0].header['PIXVALUE'], 0)

        assert_raises(KeyError, lambda: hdul1[0].header['NPIX1'])
        assert_raises(KeyError, lambda: hdul1[0].header['NPIX2'])

        assert_true((hdul1[0].data ==
                     np.zeros((10, 10), dtype=np.int32)).all())

        assert_equal(hdul1[1].header['NAXIS'], 2)
        assert_equal(hdul1[1].header['NAXIS1'], 10)
        assert_equal(hdul1[1].header['NAXIS2'], 10)
        assert_equal(hdul1[1].header['PIXVALUE'], 2)

        assert_raises(KeyError, lambda: hdul1[1].header['NPIX1'])
        assert_raises(KeyError, lambda: hdul1[1].header['NPIX2'])

        assert_true((hdul1[1].data ==
                     (np.zeros((10, 10), dtype=np.int32) + 2)).all())

        assert_equal(hdul2[0].header['NAXIS'], 0)
        assert_equal(hdul2[0].header['NPIX1'], 10)
        assert_equal(hdul2[0].header['NPIX2'], 10)
        assert_equal(hdul2[0].header['PIXVALUE'], 0)

        assert_raises(KeyError, lambda: hdul2[0].header['NAXIS1'])
        assert_raises(KeyError, lambda: hdul2[0].header['NAXIS2'])

        assert_equal(hdul2[0].data, None)

        assert_equal(hdul2[1].header['NAXIS'], 0)
        assert_equal(hdul2[1].header['NPIX1'], 10)
        assert_equal(hdul2[1].header['NPIX2'], 10)
        assert_equal(hdul2[1].header['PIXVALUE'], 2)

        assert_raises(KeyError, lambda: hdul2[1].header['NAXIS1'])
        assert_raises(KeyError, lambda: hdul2[1].header['NAXIS2'])

        hdul1.close()
        hdul2.close()

    def testHDUList__getitem__Method(self):
        """Test the __getitem__ method of st_HDUList in the stpyfits
           namespace."""

        n = np.zeros(10)
        n = n + 1

        hdu = stpyfits.PrimaryHDU(n)
        hdu.header.set('PIXVALUE', 1., 'constant pixel value', after='EXTEND')

        hdu.writeto(self.temp('new.fits'), clobber=True)

        hdul = stpyfits.open(self.temp('new.fits'))
        hdul1 = pyfits.open(self.temp('new.fits'))

        hdu = hdul[0]
        hdu1 = hdul1[0]

        assert_equal(hdu.header['NAXIS'], 1)
        assert_equal(hdu.header['NAXIS1'], 10)
        assert_equal(hdu.header['PIXVALUE'], 1.0)

        assert_raises(KeyError, lambda: hdu.header['NPIX1'])

        assert_true((hdu.data == np.ones(10, dtype=np.float32)).all())
        assert_equal(hdu1.header['NAXIS'], 0)
        assert_equal(hdu1.header['NPIX1'], 10)
        assert_equal(hdu1.header['PIXVALUE'], 1.0)

        assert_raises(KeyError, lambda: hdu1.header['NAXIS1'])

        assert_equal(hdu1.data, None)

        hdul.close()
        hdul1.close()

    def testHDUListFlushMethod(self):
        """Test the flush method of HDUList in both the pyfits and stpyfits
           namespace."""

        hdu = stpyfits.PrimaryHDU()
        hdu1 = stpyfits.ImageHDU()
        hdu.data = np.zeros((10, 10), dtype=np.int32)
        hdu1.data = hdu.data + 2
        hdu.header.set('BITPIX', 32)
        hdu1.header.set('BITPIX', 32)
        hdu.header.set('NAXIS', 2)
        hdu.header.set('NAXIS1', 10, 'length of constant array axis 1',
                       after='NAXIS')
        hdu.header.set('NAXIS2', 10, 'length of constant array axis 2',
                       after='NAXIS1')
        hdu.header.set('PIXVALUE', 0, 'Constant pixel value')
        hdu1.header.set('PIXVALUE', 2, 'Constant pixel value', after='GCOUNT')
        hdu1.header.set('NAXIS', 2)
        hdu1.header.set('NAXIS1', 10, 'length of constant array axis 1',
                        after='NAXIS')
        hdu1.header.set('NAXIS2', 10, 'length of constant array axis 2',
                        after='NAXIS1')
        hdul = stpyfits.HDUList([hdu, hdu1])
        hdul.writeto(self.temp('new.fits'), clobber=True)

        hdul = stpyfits.open(self.temp('new.fits'), 'update')
        d = np.arange(10, dtype=np.int32)
        d = d * 0
        d = d + 3
        hdul[0].data = d
        hdul.flush()
        hdul.close()

        assert_equal(stpyfits.info(self.temp('new.fits'), output=False),
            [(0, 'PRIMARY', 'PrimaryHDU', 6, (10,), 'int32', ''),
             (1, '', 'ImageHDU', 8, (10, 10), 'int32', '')])
        assert_equal(pyfits.info(self.temp('new.fits'), output=False),
            [(0, 'PRIMARY', 'PrimaryHDU', 6, (), 'int32', ''),
             (1, '', 'ImageHDU', 8, (), 'int32', '')])

        hdul1 = stpyfits.open(self.temp('new.fits'))
        hdul2 = pyfits.open(self.temp('new.fits'))

        assert_equal(hdul1[0].header['NAXIS'], 1)
        assert_equal(hdul1[0].header['NAXIS1'], 10)
        assert_equal(hdul1[0].header['PIXVALUE'], 3)

        assert_raises(KeyError, lambda: hdul1[0].header['NPIX1'])

        assert_true((hdul1[0].data ==
                     (np.zeros(10, dtype=np.int32) + 3)).all())

        assert_equal(hdul2[0].header['NAXIS'], 0)
        assert_equal(hdul2[0].header['NPIX1'], 10)
        assert_equal(hdul2[0].header['PIXVALUE'], 3)

        assert_raises(KeyError, lambda: hdul2[0].header['NAXIS1'])

        assert_equal(hdul2[0].data, None)

        hdul1.close()
        hdul2.close()

        hdul3 = stpyfits.open(self.temp('new.fits'), 'update')
        d = np.arange(15, dtype=np.int32)
        d = d * 0
        d = d + 4
        hdul3[0].data = d
        hdul3.close()      # Note that close calls flush

        assert_equal(stpyfits.info(self.temp('new.fits'), output=False),
            [(0, 'PRIMARY', 'PrimaryHDU', 6, (15,), 'int32', ''),
             (1, '', 'ImageHDU', 8, (10, 10), 'int32', '')])
        assert_equal(pyfits.info(self.temp('new.fits'), output=False),
            [(0, 'PRIMARY', 'PrimaryHDU', 6, (), 'int32', ''),
             (1, '', 'ImageHDU', 8, (), 'int32', '')])

        hdul1 = stpyfits.open(self.temp('new.fits'))
        hdul2 = pyfits.open(self.temp('new.fits'))

        assert_equal(hdul1[0].header['NAXIS'], 1)
        assert_equal(hdul1[0].header['NAXIS1'], 15)
        assert_equal(hdul1[0].header['PIXVALUE'], 4)

        assert_raises(KeyError, lambda: hdul1[0].header['NPIX1'])

        assert_true((hdul1[0].data ==
                     (np.zeros(15, dtype=np.int32) + 4)).all())

        assert_equal(hdul2[0].header['NAXIS'], 0)
        assert_equal(hdul2[0].header['NPIX1'], 15)
        assert_equal(hdul2[0].header['PIXVALUE'], 4)

        assert_raises(KeyError, lambda: hdul2[0].header['NAXIS1'])

        assert_equal(hdul2[0].data, None)

        hdul1.close()
        hdul2.close()

    def testImageBaseHDU__getattr__Method(self):
        """Test the __getattr__ method of ImageBaseHDU in both the pyfits
           and stpyfits namespace."""

        hdul = stpyfits.open(self.data('cdva2.fits'))
        hdul1 = pyfits.open(self.data('cdva2.fits'))

        hdu = hdul[0]
        hdu1 = hdul1[0]

        assert_true((hdu.data == np.ones((10, 10), dtype=np.int32)).all())
        assert_equal(hdu1.data, None)

        hdul.close()
        hdul1.close()

    def testImageBaseHDUWriteToMethod(self):
        """Test the writeto method of _ConstantValueImageBaseHDU in the
        stpyfits namespace."""

        n = np.zeros(10)
        n = n + 1

        hdu = stpyfits.PrimaryHDU(n)
        hdu.header.set('PIXVALUE', 1., 'constant pixel value', after='EXTEND')

        hdu.writeto(self.temp('new.fits'), clobber=True)

        hdul = stpyfits.open(self.temp('new.fits'))
        hdul1 = pyfits.open(self.temp('new.fits'))

        assert_equal(hdul[0].header['NAXIS'], 1)
        assert_equal(hdul[0].header['NAXIS1'], 10)
        assert_equal(hdul[0].header['PIXVALUE'], 1.0)

        assert_raises(KeyError, lambda: hdul[0].header['NPIX1'])

        assert_true((hdul[0].data == np.ones(10, dtype=np.float32)).all())

        assert_equal(hdul1[0].header['NAXIS'], 0)
        assert_equal(hdul1[0].header['NPIX1'], 10)
        assert_equal(hdul1[0].header['PIXVALUE'], 1.0)

        assert_raises(KeyError, lambda: hdul1[0].header['NAXIS1'])

        assert_equal(hdul1[0].data, None)

        hdul.close()
        hdul1.close()



"""Text output-related utilities."""


import textwrap


def wrap(text, width, *args, **kwargs):
    """
    Like :func:`textwrap.wrap` but preserves existing newlines which
    :func:`textwrap.wrap` does not otherwise handle well.

    See Also
    --------
    :func:`textwrap.wrap`
    """

    return sum([textwrap.wrap(line, width, *args, **kwargs)
                if line else [''] for line in text.splitlines()], [])


def textbox(text, width=78, boxchar='#'):
    """
    Outputs line-wrapped text wrapped in a box drawn with a repeated (usually
    ASCII) character.

    For example:

        >>> textbox('Text to wrap', width=16)
        ################
        #              #
        # Text to wrap #
        #              #
        ################

    Parameters
    ----------
    text : string
        The text to wrap

    width : int
        The width of the entire box, including the perimeter.  Because the
        wrapped text is padded with an additional column of whitespace on each
        side, the minimum width is 5--any width less than that is
        is automatically increased to 5 (default: 78)

    boxchar : string
        (No pun intended.) The character to draw the box with.  May also
        be a string of multiple characters (default: '#')

    """

    min_width = len(boxchar) * 2 + 3
    width = max(width, min_width)

    wrap_width = width - min_width + 1

    q, r = divmod(width, len(boxchar))
    # The top/bottom border
    top_border = boxchar * q + boxchar[:r]
    top_padding  = boxchar + ' ' * (width - len(boxchar) * 2) + boxchar

    lines = ['%s %s %s' % (boxchar, line.ljust(wrap_width), boxchar)
             for line in wrap(text, wrap_width)]
    top = [top_border, top_padding]
    bottom = [top_padding, top_border]
    return '\n'.join(top + lines + bottom)
